@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# Copyright 2005-2006 Messiah College. All rights reserved.
+# Copyright 2005-2007 Messiah College. All rights reserved.
 # Jason Long <jlong@messiah.edu>
 
 # Copyright (c) 2004 Anthony D. Urso. All rights reserved.
@@ -10,7 +10,6 @@
 use strict;
 use warnings;
 
-use Mail::DKIM::PrivateKey;
 use Mail::DKIM::Canonicalization::nowsp;
 use Mail::DKIM::Canonicalization::relaxed;
 use Mail::DKIM::Canonicalization::simple;
@@ -35,9 +34,6 @@ sub init
 	croak "no signature" unless $self->{Signature};
 
 	$self->{mode} = $self->{Signature}->signature ? "verify" : "sign";
-	$self->{draft_version} ||=
-		($self->{mode} eq "sign" ? "01" :
-		 $self->{Signature}->body_hash ? "01" : "00");
 
 	# allows subclasses to set the header_digest and body_digest
 	# properties
@@ -50,12 +46,10 @@ sub init
 	my $body_class = $self->get_canonicalization_class($body_method);
 	$self->{canon} = $header_class->new(
 			output_digest => $self->{header_digest},
-			draft_version => $self->{draft_version},
 			Signature => $self->{Signature},
 			Debug_Canonicalization => $self->{Debug_Canonicalization});
 	$self->{body_canon} = $body_class->new(
 			output_digest => $self->{body_digest},
-			draft_version => $self->{draft_version},
 			Signature => $self->{Signature},
 			Debug_Canonicalization => $self->{Debug_Canonicalization});
 }
@@ -186,6 +180,10 @@ sub check_body_hash
 {
 	my $self = shift;
 
+	# The body_hash value is set in finish_message(), if we're operating
+	# from a version of the DKIM spec that uses the bh= tag. Otherwise,
+	# the signature shouldn't have a bh= tag to check.
+
 	if ($self->{body_hash})
 	{
 		my $body_hash = $self->{body_hash};
@@ -206,33 +204,27 @@ sub check_body_hash
 sub finish_message
 {
 	my $self = shift;
-#	$self->{canon}->finish_message;
 
-	if ($self->{draft_version} eq "01")
+	# DKIM requires the signature itself to be committed into the digest.
+	# But first, we need to set the bh= tag on the signature, then
+	# "prettify" it.
+
+	$self->{body_hash} = $self->{body_digest}->digest;
+	if ($self->{mode} eq "sign")
 	{
-		$self->{body_hash} = $self->{body_digest}->digest;
-		if ($self->{mode} eq "sign")
-		{
-			$self->{Signature}->body_hash(
-					encode_base64($self->{body_hash}, ""));
-		}
-	#	else
-	#	{
-	#		print STDERR "verify: body hash is "
-	#			. encode_base64($self->{body_hash}, "") . "\n";
-	#	}
+		$self->{Signature}->body_hash(
+				encode_base64($self->{body_hash}, ""));
+	}
+
+	if ($self->{mode} eq "sign")
+	{
+		$self->{Signature}->prettify;
 	}
 
 	my $sig_line = $self->{Signature}->as_string_without_data;
 	my $canonicalized = $self->{canon}->canonicalize_header($sig_line);
 
-	if ($self->{draft_version} eq "00")
-	{
-		$canonicalized = "\015\012" . $canonicalized;
-	}
-
 	$self->{canon}->output($canonicalized);
-	#$self->{header_digest}->add($canonicalized);
 }
 
 =head2 sign() - generates a signature using a private key
@@ -283,3 +275,23 @@ sub verify
 }
 
 1;
+
+__END__
+
+=head1 SEE ALSO
+
+L<Mail::DKIM>
+
+=head1 AUTHOR
+
+Jason Long, E<lt>jlong@messiah.eduE<gt>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2005-2007 by Messiah College
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.6 or,
+at your option, any later version of Perl 5 you may have available.
+
+=cut
