@@ -10,8 +10,6 @@
 use strict;
 use warnings;
 
-use Mail::DKIM::PrivateKey;
-
 package Mail::DKIM::Algorithm::rsa_sha1;
 use base "Mail::DKIM::Algorithm::Base";
 use Carp;
@@ -24,14 +22,7 @@ sub init_digests
 
 	# initialize a SHA-1 Digest
 	$self->{header_digest} = Digest::SHA->new(1);
-	if ($self->{draft_version} eq "01")
-	{
-		$self->{body_digest} = Digest::SHA->new(1);
-	}
-	else
-	{
-		$self->{body_digest} = $self->{header_digest};
-	}
+	$self->{body_digest} = Digest::SHA->new(1);
 }
 
 sub sign
@@ -41,7 +32,7 @@ sub sign
 	my ($private_key) = @_;
 
 	my $digest = $self->{header_digest}->digest;
-	my $signature = $private_key->sign_sha1_digest($digest);
+	my $signature = $private_key->sign_digest("SHA-1", $digest);
 
 	return encode_base64($signature, "");
 }
@@ -56,7 +47,7 @@ sub verify
 
 	my $sig = decode_base64($base64);
 	my $digest = $self->{header_digest}->digest;
-	return unless $public_key->verify_sha1_digest($digest, $sig);
+	return unless $public_key->verify_digest("SHA-1", $digest, $sig);
 	return $self->check_body_hash;
 }
 
