@@ -111,7 +111,7 @@ is written to the referenced string or file handle.
 package Mail::DKIM::Verifier;
 use base "Mail::DKIM::Common";
 use Carp;
-our $VERSION = 0.37;
+our $VERSION = 0.38;
 
 sub init
 {
@@ -157,6 +157,7 @@ sub handle_header
 		{
 			my $signature = Mail::DKIM::Signature->parse($line);
 			$self->add_signature($signature);
+			$signature->fetch_public_key;
 		};
 		if ($@)
 		{
@@ -176,6 +177,7 @@ sub handle_header
 		{
 			my $signature = Mail::DKIM::DkSignature->parse($line);
 			$self->add_signature($signature);
+			$signature->fetch_public_key;
 		};
 		if ($@)
 		{
@@ -407,29 +409,6 @@ sub finish_header
 				$self->{signature_reject_reason});
 			next;
 		}
-
-		# get public key
-		my $pkey;
-		eval
-		{
-			$pkey = $signature->get_public_key;
-		};
-		if ($@)
-		{
-			my $E = $@;
-			chomp $E;
-			$self->{signature_reject_reason} = "public key: $E";
-			$signature->result("invalid",
-				$self->{signature_reject_reason});
-			next;
-		}
-
-		unless ($self->check_public_key($signature, $pkey))
-		{
-			$signature->result("invalid",
-				$self->{signature_reject_reason});
-			next;
-		}
 	}
 
 	# stop processing signatures that are already known to be invalid
@@ -448,14 +427,31 @@ sub finish_header
 	}
 }
 
-sub finish_body
+sub _check_and_verify_signature
 {
 	my $self = shift;
+	my ($algorithm) = @_;
 
-	foreach my $algorithm (@{$self->{algorithms}})
-	{
-		# finish canonicalizing
-		$algorithm->finish_body;
+		# check signature
+		my $signature = $algorithm->signature;
+		# get public key
+		my $pkey;
+		eval
+		{
+			$pkey = $signature->get_public_key;
+		};
+		if ($@)
+		{
+			my $E = $@;
+			chomp $E;
+			$self->{signature_reject_reason} = "public key: $E";
+			return ("invalid", $self->{signature_reject_reason});
+		}
+
+		unless ($self->check_public_key($signature, $pkey))
+		{
+			return ("invalid", $self->{signature_reject_reason});
+		}
 
 		# verify signature
 		my $result;
@@ -481,6 +477,19 @@ sub finish_body
 			$result = "fail";
 			$details = $E;
 		}
+	return ($result, $details);
+}
+
+sub finish_body
+{
+	my $self = shift;
+
+	foreach my $algorithm (@{$self->{algorithms}})
+	{
+		# finish canonicalizing
+		$algorithm->finish_body;
+
+		my ($result, $details) = $self->_check_and_verify_signature($algorithm);
 
 		# save the results of this signature verification
 		$algorithm->{result} = $result;
@@ -512,6 +521,12 @@ Feeds content of the message being verified into the verifier.
 The API is designed this way so that the entire message does NOT need
 to be read into memory at once.
 
+Please note that although the PRINT() method expects you to use
+SMTP-style line termination characters, you should NOT use the
+SMTP-style dot-stuffing technique described in RFC 2821 section 4.5.2.
+Nor should you use a <CR><LF>.<CR><LF> sequence to terminate the
+message.
+
 =head2 CLOSE()
 
 Call this when finished feeding in the message.
